class version:
    package = '0.1.7 beta'
    InterfacesInfos = '1.2'
    InterfacesRoutes = '1.2'
    IfaceInfoTools = '0.6'
    FileReader = '0.2'
    Conn = '1.0'
    TCPConn = '1.2'
    UDPConn = '1.2'